--------------------------------------------------------------------------------
-- |
-- Module      :  Graphics.Rendering.OGL.GL.Antialiasing
-- Copyright   :  (c) Sven Panne 2002-2006
-- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
-- 
-- Maintainer  :  sven.panne@aedion.de
-- Stability   :  stable
-- Portability :  portable
--
-- This module corresponds to section 3.2 (Antialiasing) of the OpenGL 2.1
-- specs.
--
--------------------------------------------------------------------------------

module Graphics.Rendering.OGL.GL.Antialiasing (
   sampleBuffers, samples, multisample, subpixelBits
) where
import Graphics.Rendering.OGL.Monad
import Graphics.Rendering.OGL.GL.Capability (
   EnableCap(CapMultisample), makeCapability )
import Graphics.Rendering.OGL.GL.BasicTypes ( GLsizei, Capability )
import Graphics.Rendering.OGL.GL.QueryUtils (
   GetPName(GetSampleBuffers,GetSamples,GetSubpixelBits), getSizei1 )
import Graphics.Rendering.OGL.GL.StateVar (
   GettableStateVar, makeGettableStateVar, StateVar )

--------------------------------------------------------------------------------

sampleBuffers :: GettableStateVar GLsizei
sampleBuffers = antialiasingInfo GetSampleBuffers

samples :: GettableStateVar GLsizei
samples = antialiasingInfo GetSamples

multisample :: StateVar Capability
multisample = makeCapability CapMultisample

subpixelBits :: GettableStateVar GLsizei
subpixelBits = antialiasingInfo GetSubpixelBits

antialiasingInfo :: GetPName -> GettableStateVar GLsizei
antialiasingInfo = makeGettableStateVar . getSizei1 id
